<?php

require_once "ast/AddExpression.inc";
require_once "ast/DivExpression.inc";
require_once "ast/MulExpression.inc";
require_once "ast/NumberExpression.inc";
require_once "ast/SubExpression.inc";
require_once "ast/UnaryExpression.inc";
require_once "ast/UnaryMinusExpression.inc";

class Parser
{
	var $token;
	var $scanner;
	
	function Parser( $scanner )
	{
		$this->scanner = $scanner;
	}
	
	function parse()
	{
		$this->token = $this->scanner->nextToken();
		$expr = $this->parseExpression();
		
		if( $this->token->type == TOKEN_TYPE_EOF )
		{
			return $expr;
		}else
		{
			ntxMsgErr( "Errore in ".$this->token->toString() );
		}
	}
	
	function parseExpression()
	{
		$left = $this->parseTerm();
		
		while( ( $this->token->type == TOKEN_TYPE_ADD ) || ( ( $this->token->type == TOKEN_TYPE_SUB ) ) )
		{
			$operator = $this->token->type;
			$this->token = $this->scanner->nextToken();
			$right = $this->parseTerm();
			
			if( $operator == TOKEN_TYPE_ADD )
			{
				$left = new AddExpression( $left, $right );
			}else
			{
				$left = new SubExpression( $left, $right );
			}
		}
		
		return $left;
	}
	
	function parseTerm()
	{
		$left = $this->parseFactor();
		
		while( ( $this->token->type == TOKEN_TYPE_MUL ) || ( ( $this->token->type == TOKEN_TYPE_DIV ) ) )
		{
			$operator = $this->token->type;
			$this->token = $this->scanner->nextToken();
			$right = $this->parseFactor();
			
			if( $operator == TOKEN_TYPE_MUL )
			{
				$left = new MulExpression( $left, $right );
			}else
			{
				$left = new DivExpression( $left, $right );
			}
		}
		
		return $left;
	}
	
	function parseFactor()
	{
		$unary = array();
		
		while( ( $this->token->type == TOKEN_TYPE_ADD ) || ( ( $this->token->type == TOKEN_TYPE_SUB ) ) )
		{
			array_push( $unary, $this->token );
			$this->token = $this->scanner->nextToken();
		}
		
		switch( $this->token->type )
		{
			case TOKEN_TYPE_NUM:
				$tree = new NumberExpression( $this->token->value );
				$this->token = $this->scanner->nextToken();
				break;
						
			case TOKEN_TYPE_LEFT_PAR:
				$this->token = $this->scanner->nextToken();
				$tree = $this->parseExpression();
				
				if( $this->token->type == TOKEN_TYPE_RIGHT_PAR )
				{
					$this->token = $this->scanner->nextToken();
				}else
				{
					ntxMsgErr( "Errore in chiusura parentesi" );
				}
				
				break;
			
			default:
				ntxMsgErr( "Errore in ".$this->token->toString() );
				break;
				
		}
		
		while( count( $unary ) > 0 )
		{
			$tokent = array_pop( $unary );
			
			if( $tokent->type == TOKEN_TYPE_ADD )
			{
				$tree = new UnaryPlusExpression( $tree );
			}else
			{
				$tree = new UnaryMinusExpression( $tree );
			}
		}
		
		return $tree;
	}

}

?>