<?php

require_once "Token.inc";
require_once "TokenType.inc";

class Scanner
{
	var $pos;
	var $source;
	
	function Scanner( $source )
	{
		$this->pos = 0;
		$this->source = $source;
	}
	
	function nextToken()
	{
		$this->skipWhites();
		
		if( $this->isEOF() )
		{
			return new Token( TOKEN_TYPE_EOF );
		}
		
		$c = $this->source{ $this->pos++ };
		
		switch( $c )
		{
			case '+':	return new Token( TOKEN_TYPE_ADD, '+' );
			case '-':	return new Token( TOKEN_TYPE_SUB, '-' );
			case '*':	return new Token( TOKEN_TYPE_MUL, '*' );
			case '/':	return new Token( TOKEN_TYPE_DIV, '/' );
			case '(':	return new Token( TOKEN_TYPE_LEFT_PAR, '(' );
			case ')':	return new Token( TOKEN_TYPE_RIGHT_PAR, ')' );
			
			default:
				
				$buf = "";
				$code = ord( $c );
				
				if( $this->isNumber( $code ) )
				{
					while( $this->isNumber( $code ) )
					{
						$buf .= $c;
						
						if( $this->isEOF() )
						{
							++$this->pos;
							break;
						}
						
						$c  = $this->source{ $this->pos++ };
						$code = ord( $c );
					}
					
					if( $c == '.' || $c == ',' )
					{
						$buf .= '.';
						$c  = $this->source{ $this->pos++ };
						$code = ord( $c );
						
						while( $this->isNumber( $code ) )
						{
							$buf .= $c;
							
							if( $this->isEOF() )
							{
								++$this->pos;
								break;
							}
							
							$c  = $this->source{ $this->pos++ };
							$code = ord( $c );
						}
						
						$num = floatval( $buf );
					}else
					{
						$num = floatval( $buf.".0" );
					}
					
					--$this->pos;
					
					return new Token( TOKEN_TYPE_NUM, $num );
				}
				
				break;
		}
		
		return new Token( TOKEN_TYPE_NULL, $c );
	}
	
	function isNumber( $c )
	{
		return ( $c >= 48 ) && ( $c <= 57 );
	}
	
	function isEOF()
	{
		return ( $this->pos >= strlen( $this->source ) );
	}
	
	function skipWhites()
	{
		while( !$this->isEOF() )
		{
			$c = $this->source{ $this->pos++ };
			
			if( ( $c != " " ) && ( $c != "\t" ) )
			{
				--$this->pos;
				break;
			}
		}
	}
}

?>