<?php
/**
* classe statica che permette di generare e validare token
*
* la classe offre solo metodi statici-
* con getToken() permette di generare un token associato all'id di sessione univoco dell'utente.
* con checkToken($token) verifichiamo che il token appartenga effettivamente all'utente che ha fatto una richiesta.
* con useToken($token) permette di marchiare come utilizzato un token valido
* 
* a causa dei limiti con le classi del del linguaggio php4, le seguenti vatiabili che dovrebbero essere proprietà statiche vengono create in ogni funzione.
* $MAX_LIFE_TIME - intero. tempo di vita in minuti per un token. dopo $MAX_LIFE_TIME minuti il token non è più valido
* $PHPSESSID - stringa con id di sessione 
* $PHPSESSID_s - stringa con id di sessione  pronto per essere usato in una query
* $Utente_s - stringa con username dell'utente loggato  pronto per essere usato in una query 
* 
* 
* @package sicurezza
* @author Alessandro Zagatti
* @version 1.0 
* @copyright Natisoft srl
* @uses subclass sets a temporary variable
*/
class ntxSecurityToken
{  
    /**
     * metodo statico che ritorna il messaggio di errore di default
     * @return stringa con il messaggio di errore di default 
     */ 	
	function getStringTokenError()
	{
		return "errore!\n\npossibili cause: \n- si e' superato il maximum execution time \n- non e' stato eseguito l'accesso al portale \n";
	}
 
 
    /**
     * metodo statico che permette di generare un token associato all'id di sessione univoco dell'utente. 
     * @return stringa con il token 
     */ 
	function getToken()
	{  
		$token = sha1(uniqid(rand(),TRUE));
		
		$token_s = ntxs($token);	 
				
		$MAX_LIFE_TIME	= 20;   
		$PHPSESSID = "".session_id();    	 	
		
		$sql = <<<_SQL
		DELETE FROM 
			UtentiToken 
		WHERE  
			UtentiToken.token = $token_s  
_SQL;
		ntxQuery($sql);
		
		
		$UtentiToken= new NtxTable("UtentiToken");		
		$insert= &$UtentiToken->insert(); 
		 
		$insert->set("PHPSESSID", $PHPSESSID );
		$insert->set("token", $token );
		$insert->set("data", "GETDATE()", true);        
		$insert->set("utilizzato", 0);        

		$insert->execute();   
		
		return $token;
	}

    /**
     * metodo statico che permette di verificare che un token sia stato richiesto nell'ambito della sessione corrente
     * @param string $token con il token da validare
     * @return boolen con il risultato della verifica 
     */ 	
	function checkToken($token)
	{
		$ritorno = false; 
		
		$token_s = ntxs($token);	
	 
		$MAX_LIFE_TIME	= 30;   
		$PHPSESSID = "".session_id();    		
		$PHPSESSID_s = ntxs($PHPSESSID);   		
		$Utente_s = ntxs("".$_SESSION['Utente']);  		
		
	 
		if(strlen($token)>0)  
		{ 
			$sql = <<<_SQL
			SELECT TOP 1 UtentiToken.ID
			FROM 
				UtentiToken 
				INNER JOIN
				UtentiSessione ON UtentiToken.PHPSESSID = UtentiSessione.PHPSESSID			
			
			WHERE 
				UtentiSessione.Utente = $Utente_s
				AND UtentiSessione.PHPSESSID = $PHPSESSID_s	
				AND UtentiSessione.valido = 1
				 
				AND UtentiToken.utilizzato = 0
				
				AND UtentiToken.token = $token_s
				AND DATEDIFF(minute, UtentiToken.data, GETDATE()) <= $MAX_LIFE_TIME
				
			ORDER BY UtentiToken.Data Desc		
_SQL;
			$rs = ntxRecord(ntxQuery($sql));
			$IdCheck1 = $rs['ID'];
	
			if(is_numeric($IdCheck1))
				$ritorno = true;	
		} 
 
		return $ritorno;
	}  
	
	/**
	* metodo statico che permette di marchiare come utilizzato un token associato all'id di sessione univoco dell'utente. 
	* @param string $token con il token da invalidare
	*/ 
	function useToken($token)
	{  
		$token_s = ntxs($token);	
	 
 
		$PHPSESSID = "".session_id();    		
		$PHPSESSID_s = ntxs($PHPSESSID);   		
		$Utente_s = ntxs("".$_SESSION['Utente']);  		
		
	 
		if(strlen($token)>0)  
		{ 
			$sql = <<<_SQL
			UPDATE   UtentiToken
			SET   utilizzato = 1

			FROM 
				UtentiToken 
				INNER JOIN
				UtentiSessione ON UtentiToken.PHPSESSID = UtentiSessione.PHPSESSID			

			WHERE 
				UtentiSessione.Utente = $Utente_s
				AND UtentiSessione.PHPSESSID = $PHPSESSID_s	
				AND UtentiSessione.valido = 1
				 
				AND UtentiToken.utilizzato = 0
				
				AND UtentiToken.token = $token_s  	
_SQL;
			ntxQuery($sql);
		}	 
	}	
}
?>