<?php
/**
* Generatore di file excel in formato XML per PHP4 o superiore
* 
* Usando le specifiche Microsoft XML viene creato un documento XML nativo che puo' essere aperto ed editato da Excel 
* 
* @version 1.0
*   
* ********************
* * ESEMPIO DI USO: **
* ********************
*
* require 'php-excel.class.php'; 
* 
* 
* $MyDefaulStyle = '
* 	<Style ss:ID="Default" ss:Name="Normal">
* 		<Alignment ss:Vertical="Bottom"/>
* 		<Borders/>
* 		<Font ss:FontName="Calibri" x:Family="Swiss" ss:Size="11" ss:Color="#000000"/>
* 		<Interior />
* 		<NumberFormat/>
* 		<Protection/>
* 	</Style>
* 
* 	<Style ss:ID="stileGrosso">
* 		<Font x:Family="Swiss" ss:Bold="1" />
* 	</Style>		
* ';		
* 	
* $fogli = array();		
* $fogli[0] = "pagina 0";	
* $fogli[1] = "pagina 1";		
* 
* $xls = new Excel_XML('UTF-8', false, $fogli); 
* 
* 
* $xls->setStyles($MyDefaulStyle);
* 
* $xls->addRow( 0, array ('(1 1)', '(1 2)', '(1 3)'), array('ss:StyleID="stileGrosso"' , 'ss:StyleID="Normal"') );
* $xls->addRow( 2, array ('(2 1)', '(2 2)', '(2 3)') );
* 
* 
* $xls->generateXML('my-test');
*   
*   
*/
class Excel_XML
{
	/**
	* Header del documento XML secondo le specifiche di microsoft 
	*/
	var $header = '<?xml version="1.0"?><?mso-application progid="Excel.Sheet"?><Workbook xmlns="urn:schemas-microsoft-com:office:spreadsheet" xmlns:o="urn:schemas-microsoft-com:office:office" xmlns:x="urn:schemas-microsoft-com:office:excel" xmlns:ss="urn:schemas-microsoft-com:office:spreadsheet" xmlns:html="http://www.w3.org/TR/REC-html40">';

	/**
	* Variabile che contiene la dichiarazione degli stili
	*/	
	var $styles ;

	/**
	* Footer 
	*/
	var $footer = "</Workbook>";

	/**
	* column in the excel document
	*/
	var $column = array();	
	
	/**
	* Lines to output in the excel document
	*/
	var $lines = array();

	/**
	* Used encoding
	*/
	var $sEncoding;

	/**
	* Convert variable types
	*/
	var $bConvertTypes;

	/**
	* Array con gli Worksheet title
	*/
	var $sWorksheetTitle ;

	
	/**
	 * Constructor
	 *
	 * Il costruttore permette di configurare parametri addizionali della libreria.
	 *
	 * 
	 * Per convertire i tipi:
	 * 	quando il secondo parametro è settato a true la libreria prova a indentificar e convertire il tipo nel rispettivo tipo excel.
	 * 
	 * @param string $sEncoding Encoding che deve essere usato (defaults UTF-8)
	 * @param boolean $bConvertTypes converte le variabili al tipo specificato
	 * @param string $arrayWorksheetTitle array con i worksheet titles  
	 */
	 function Excel_XML($sEncoding = 'UTF-8', $bConvertTypes = false, $arrayWorksheetTitle)
	{
		$this->bConvertTypes = $bConvertTypes;
		$this->setEncoding($sEncoding);
		
		$this->sWorksheetTitle = array();
		
		for($i=0; $i<count($arrayWorksheetTitle); $i++)
		{
			$tmp = $arrayWorksheetTitle[$i]."";
			$tmp = str_replace(" ", "", $tmp );
		
			$this->sWorksheetTitle[$i] = $this->setWorksheetTitle( "[$i]".$tmp );	
		}
	}
	
	/**
	 * Set encoding
	 * @param string Encoding type to set
	 */
	 function setEncoding($sEncoding)
	{
		$this->sEncoding = $sEncoding;
	}

	/**
	 * Set worksheet title
	 * 
	 * Esegue le operazioni necessarie perche' il titolo del foglio di lavoro sia valido
	 * 
	 * @param string $title Title for worksheet
	 */
	 function setWorksheetTitle ($title)
	{
			$title = preg_replace ("/[\\\|:|\/|\?|\*|\[|\]]/", "", $title);
			$title = substr ($title, 0, 31);
			
			if($title.""=="")
				$title="unknown";
			
			return $title;
	}

	
	
	/**
	 * Set document styles
	 * 
	 * Permette di aggiungere uno o più stili al documento XML
	 * 
	 * @param string $newStyle string with additional styles
	 */	
	function setStyles($newStyle)
	{
		$this->styles .= '<Styles>';
		
		$this->styles .= $newStyle;
		
		$this->styles .= '</Styles>';
	}		

	/**
	 * Add Column 
	 * 
	 * Adds a single Column  to the document. 
	 * 
	 * @param array $sheet worksheet when print the row
	 * @param array $Width column-width
	 * @param array $AutoFitWidth enable/disabled column auto fit
	 */
	function addColumn($sheet , $Width=null, $AutoFitWidth=null)
	{
		if(is_numeric($Width))
			$customWidth = " ss:Width=\"$Width\" ";			
		else
			$customWidth = "";
			
		if(is_numeric($AutoFitWidth))
			$customAutoFitWidth = " ss:AutoFitWidth=\"$AutoFitWidth\" ";			
		else
			$customAutoFitWidth = "";			
	
		$this->column[$sheet][] = "<Column  $customAutoFitWidth $customWidth />\n";
	}	
	
	
	
	/**
	 * Add row
	 * 
	 * Adds a single row to the document. If set to true, self::bConvertTypes
	 * checks the type of variable and returns the specific field settings
	 * for the cell.
	 * 
	 * @param array $sheet worksheet when print the row
	 * @param array $array One-dimensional array with row content
	 * @param array $attributes attributes as style, width, ecc...
	 */
	function addRow ($sheet , $array, $attributes=null, $convertHTML=true)
	{
		$cells = "";
		
		for($i=0; $i<count($array); $i++)
		{	
			$v = str_replace("  ", " ", $array[$i]."" );

			$type = 'String';
			if ($this->bConvertTypes === true  &&  is_numeric($v)):
					$type = 'Number';
			endif;
			
			
			$attribute = "";
			
			if( $attributes )
				$attribute = $attributes[$i];
			
			
			if($convertHTML)
				$v = htmlentities($v, ENT_COMPAT, $this->sEncoding);
		
			$cells .= "<Cell ".$attribute."> <Data ss:Type=\"$type\">" . $v . "</Data> </Cell>\n"; 
		}

		$this->lines[$sheet][] = "<Row  ss:AutoFitHeight=\"1\" >\n" . $cells . "</Row>\n";
	}

	/**
	 * Generate the excel file
	 * @param string $filename Name of excel file to generate (...xls)
	 */
	 function generateXML ($filename = 'excel-export')
	{
			//correct/validate filename
			$filename = preg_replace('/[^aA-zZ0-9\_\-]/', '', $filename);
	
			//deliver header (as recommended in php manual)
			header("Content-Type: application/vnd.ms-excel; charset=" . $this->sEncoding);
			header("Content-Disposition: inline; filename=\"" . $filename . ".xls\"");

			//print out document to the browser
			echo stripslashes ( sprintf($this->header."".$this->styles, $this->sEncoding) );
			
			//print the content worksheets
			for($i=0; $i<count($this->sWorksheetTitle); $i++)
			{
				echo "\n<Worksheet ss:Name=\"".$this->sWorksheetTitle[$i]."\">\n<Table>\n";
				
				foreach ($this->column[$i] as $column)
					echo $column;				
				
				foreach ($this->lines[$i] as $line)
					echo $line;

				echo "</Table>\n</Worksheet>\n";
			}			

			echo $this->footer;
	}
}

?>