/*

Original code: JC Grubbs - jc.grubbs@devmynd.com - jQuery.ganttView v.0.8.8

Developed: Alessandro Zagatti

MIT License Applies
*/

/*
Options
-----------------
showWeekends: boolean
data: object
cellWidth: number
cellHeight: number
slideWidth: number
dataUrl: string
behavior: {
	clickable: boolean,
	draggable: boolean,
	resizable: boolean,
	onClick: function,
	onDrag: function,
	onResize: function
}
*/

(function ($) {
	
    $.fn.ganttView = function () {
    	
    	var args = Array.prototype.slice.call(arguments);	
		
    	if (args.length == 1  &&  typeof(args[0]) == "object") {
        	build.call(this, args[0]);
    	}
    	
    	if (args.length == 2  &&  typeof(args[0]) == "string") {
    		handleMethod.call(this, args[0], args[1]);
    	}
    };
    
    function build(options) {
    	
    	var els = this;
        var defaults = {
            showWeekends: true,
            cellWidth: 21,
            cellHeight: 31,
            slideWidth: 400,
            vHeaderWidth: 100,
            behavior: {
            	dblclick: true,
            	clickable: true,
            	draggable: true,
            	resizable: true
            },
			interval: [false, false],
			colors: {
            	idx: 0,
            	arrayColors: new Array("#EBECE4", "#F1EDC2","#ECF1EF","#FBF5E6","#BCEE68","#E9967A","#DAA520","#CDCDC1","#F0E68C","#EEE0E5","#D3D3D3","#CAE1FF","#EED5D2","#FFFFFF")
            },
			typeView: 'days'
        };
 
        var opts = jQuery.extend(true, defaults, options);

		var ss = document.styleSheets;

		
		if( opts.typeView=='months' )
		{		
			opts.cellWidth = 3;
		}      
		
		if (opts.data) {
			build();
		} else if (opts.dataUrl) {
			jQuery.getJSON(opts.dataUrl, function (data) { opts.data = data; build(); });
		}

		function build() {
	
			
			var minDays = 1;//ORIGINAL CODE: Math.floor((opts.slideWidth / opts.cellWidth)  + 5);  		
			var startEnd = DateUtils.getBoundaryDatesFromData(opts.data, minDays);  
			opts.start = startEnd[0];
			opts.end = startEnd[1];

	
	        els.each(function () {

	            var container = jQuery(this);
	            var div = jQuery("<div>", { "class": "ganttview" });
	            new Chart(div, opts).render();
				container.append(div);
				
				var w = jQuery("div.ganttview-vtheader", container).outerWidth() +
					jQuery("div.ganttview-slide-container", container).outerWidth();
	            container.css("width", (w + 2) + "px");
	            
	            new Behavior(container, opts).apply();
	        });
		}
    }

	function handleMethod(method, value) {
		
		if (method == "setSlideWidth") {
			var div = $("div.ganttview", this);
			div.each(function () {
				var vtWidth = $("div.ganttview-vtheader", div).outerWidth();
				$(div).width(vtWidth + value + 1);
				$("div.ganttview-slide-container", this).width(value);
			});
		}			
	}

	var Chart = function(div, opts) {
		
		function render() {
			addVtHeader(div, opts.data, opts.cellHeight);

            var slideDiv = jQuery("<div>", {
                "class": "ganttview-slide-container",
                "css": { "width": opts.slideWidth + "px" }
            });
			
			//qua prendo l'array
            dates = getDates(opts.start, opts.end);
            addHzHeader(slideDiv, dates, opts.cellWidth, opts.interval);//creo i giorni
            addGrid(slideDiv, opts.data, dates, opts.cellWidth, opts.showWeekends);
            addBlockContainers(slideDiv, opts.data);
            addBlocks(slideDiv, opts.data, opts.cellWidth, opts.start, opts.end);
            div.append(slideDiv);
            applyLastClass(div.parent());
		}
		
		var monthNames = ["Gen", "Feb", "Mar", "Apr", "Mag", "Giu", "Lug", "Ago", "Set", "Ott", "Nov", "Dic"];

		// Creates a 3 dimensional array [year][month][day] of every day 
		// between the given start and end dates
        function getDates(start, end) {
            var dates = [];
			dates[start.getFullYear()] = [];
			dates[start.getFullYear()][start.getMonth()] = [start]
			var last = start;
			while (last.compareTo(end) == -1) {
				var next = last.clone().addDays(1);
				if (!dates[next.getFullYear()]) { dates[next.getFullYear()] = []; }
				if (!dates[next.getFullYear()][next.getMonth()]) { 
					dates[next.getFullYear()][next.getMonth()] = []; 
				}
				dates[next.getFullYear()][next.getMonth()].push(next);
				last = next;
			}
			return dates;
        }

        function addVtHeader(div, data, cellHeight) 
		{
            var headerDiv = jQuery("<div>", { "class": "ganttview-vtheader" });
			
	
			for (var k = 0; k < data.length; k++)  //add AZ
			{
				var itemDiv = jQuery("<div>", { "class": "ganttview-vtheader-item_0" });
				
				//troviamo tutti gli elementi della sottoserie
				var numberElem = 0;
				for (var i = 0; i < data[k].series.length; i++) 
				{
					numberElem += data[k].series[i].series.length;
									
				}			
						
				//var itemDiv = jQuery("<div>", { "class": "ganttview-vtheader-item" });
				itemDiv.append(jQuery("<div>", 
				{
					"class": "ganttview-vtheader-item-name",
					"css": { "height": (cellHeight * numberElem) + "px" }
				}).append(data[k].name));			
				
		
				
				for (var i = 0; i < data[k].series.length; i++) 
				{
					var itemDiv2 = jQuery("<div>", { "class": "ganttview-vtheader-item_1" });
					
					//var itemDiv2 = jQuery("<div>", { "class": "ganttview-vtheader-item" });
					itemDiv2.append(jQuery("<div>", 
					{
						"class": "ganttview-vtheader-item-name",
						"title": data[k].name+" << "+data[k].series[i].name,
						"css": { "height": (data[k].series[i].series.length * cellHeight) + "px" }
					}).append(data[k].series[i].name));
					
					
					var seriesDiv = jQuery("<div>", { "class": "ganttview-vtheader-series" });
					
					for (var j = 0; j < data[k].series[i].series.length; j++) 
					{
						seriesDiv.append(jQuery("<div>", { "class": "ganttview-vtheader-series-name" , "title": data[k].name+" << "+data[k].series[i].name+" << "+data[k].series[i].series[j].name })
							.append(data[k].series[i].series[j].name));
					}
					
					
					itemDiv2.append(seriesDiv);
					itemDiv.append( itemDiv2 );
					headerDiv.append(itemDiv);					
				}
			 }	 //end add AZ
			 
			div.append(headerDiv);
        }

        function addHzHeader(div, dates, cellWidth, interval) {
            var headerDiv = jQuery("<div>", { "class": "ganttview-hzheader" });
            var monthsDiv = jQuery("<div>", { "class": "ganttview-hzheader-months" });
            var daysDiv = jQuery("<div>", { "class": "ganttview-hzheader-days" });
			
            var totalW = 0;
			for (var y in dates) {
				for (var m in dates[y]) {
					var w = dates[y][m].length * cellWidth;
					totalW = totalW + w;
					monthsDiv.append(jQuery("<div>", {
						"class": "ganttview-hzheader-month",
						"css": { "width": (w - 1) + "px" }
					}).append(monthNames[m] + " " + y));
					for (var d in dates[y][m]) 
					{
						var classS = "ganttview-hzheader-day";
						
			 			if ( DateUtils.isInInterval(dates[y][m][d] , interval[0], interval[1]) ) //if is in the selected interval
						{ 
							classS = "ganttview-hzheader-day_interval"; 
						}	 
					
						if( opts.typeView=='months' )
							daysDiv.append(jQuery("<div>", { "class": classS }));
						else		
							daysDiv.append(jQuery("<div>", { "class": classS })
								.append(dates[y][m][d].getDate()));
					}
				}
			}
            monthsDiv.css("width", totalW + "px");
            daysDiv.css("width", totalW + "px");
            headerDiv.append(monthsDiv).append(daysDiv);
            div.append(headerDiv);
        }

        function addGrid(div, data, dates, cellWidth, showWeekends) 
		{
            var gridDiv = jQuery("<div>", { "class": "ganttview-grid" });
            var rowDiv = jQuery("<div>", { "class": "ganttview-grid-row" });
			for (var y in dates) 
			{
				for (var m in dates[y]) 
				{
					for (var d in dates[y][m]) 
					{
						var cellDiv = jQuery("<div>", { "class": "ganttview-grid-row-cell" });
						
						if (DateUtils.isWeekend(dates[y][m][d])  &&  showWeekends) //weekend
						{ 
							cellDiv.addClass("ganttview-weekend"); 
						}					
						
						rowDiv.append(cellDiv);
					}
				}
			}
            var w = jQuery("div.ganttview-grid-row-cell", rowDiv).length * cellWidth;
            rowDiv.css("width", w + "px");
            gridDiv.css("width", w + "px");
			
			
 			for (var k = 0; k < data.length; k++)  //add AZ
			{	
				for (var i = 0; i < data[k].series.length; i++) 
				{
					for (var j = 0; j < data[k].series[i].series.length; j++) 
					{
						gridDiv.append(rowDiv.clone());
					}
				}
			} //end add AZ 		
			
            div.append(gridDiv);
        } 

        function addBlockContainers(div, data) {
		
            var blocksDiv = jQuery("<div>", { "class": "ganttview-blocks" });
			
			for (var k = 0; k < data.length; k++)  //add AZ
			{	
				for (var i = 0; i < data[k].series.length; i++) 
				{
					for (var j = 0; j < data[k].series[i].series.length; j++) 
					{
						blocksDiv.append(jQuery("<div>", { "class": "ganttview-block-container" }));
					}
				}

			}	 //end add AZ			
			
            div.append(blocksDiv);
        }

        function addBlocks(div, data, cellWidth, start, end) {
            var rows = jQuery("div.ganttview-blocks div.ganttview-block-container", div);
            var rowIdx = 0;

			for (var k = 0; k < data.length; k++)  //add AZ
			{			
					for (var i = 0; i < data[k].series.length; i++) 
					{
						var currentColor = opts.colors.arrayColors[opts.colors.idx];
						
						if( opts.colors.idx < opts.colors.arrayColors.length-1 ) //per la scelta dei colori
							++opts.colors.idx;
						else
							opts.colors.idx = 0;		
					
					
						for (var j = 0; j < data[k].series[i].series.length; j++) 
						{
						
							for (var y = 0; y < data[k].series[i].series[j].series.length; y++) 
							{
								var series = data[k].series[i].series[j].series[y];
							
								var size = DateUtils.daysBetween(series.start, series.end) + 1;
								var offset = DateUtils.daysBetween(start, series.start) ;
								
								var block = jQuery("<div>", {
									"class": "ganttview-block",
									"title": series.name + ", " + size + " giorni",
									"css": {
										"width": ((size * cellWidth) - 9) + "px",
										"left": ((offset * cellWidth) + 3) + "px",
										"z-index": y+''+rowIdx
									}
								});
								
								addBlockData(block, series, series, start, end);
								
 								if (series.color) 
									block.css("background-color",series.color);
								else
									block.css("background-color", currentColor);	 			
								
					
								block.append(jQuery("<div>", { "class": "ganttview-block-text" }).text(size));	

								jQuery(rows[rowIdx]).append(block);							
							}
							 
							rowIdx = rowIdx + 1;
						}
						
					}
			}//end AZ			
			
        }
        
        function addBlockData(block, data, series, start, end) {
        	// This allows custom attributes to be added to the series data objects
        	// and makes them available to the 'data' argument of click, resize, and drag handlers
        	var blockData = {startRow: start.toString("M/d/yyyy"), endRow: end.toString("M/d/yyyy") }; //{ id: data.id, name: data.name };
        	jQuery.extend(blockData, series);
        	block.data("block-data", blockData);
        }

        function applyLastClass(div) {
            jQuery("div.ganttview-grid-row div.ganttview-grid-row-cell:last-child", div).addClass("last");
            jQuery("div.ganttview-hzheader-days div.ganttview-hzheader-day:last-child", div).addClass("last");
            jQuery("div.ganttview-hzheader-months div.ganttview-hzheader-month:last-child", div).addClass("last");
        }
		
		return {
			render: render
		};
	}

	var Behavior = function (div, opts) {
		
		function apply() {
			
			if (opts.behavior.dblclick) { 
            	bindBlockDbClick(div, opts.behavior.dblclick); 
        	}			
			if (opts.behavior.clickable) { 
            	bindBlockClick(div, opts.behavior.onClick); 
        	}
        	
            if (opts.behavior.resizable) { 
            	bindBlockResize(div, opts.cellWidth, opts.start, opts.behavior.onResize); 
        	}
            
            if (opts.behavior.draggable) { 
            	bindBlockDrag(div, opts.cellWidth, opts.start, opts.behavior.onDrag); 
        	}
		}

        function bindBlockClick(div, callback) {
            jQuery("div.ganttview-block", div).live("click", function () {
                if (callback) { callback(jQuery(this).data("block-data"), jQuery(this)); }
            });
        }
        
        function bindBlockDbClick(div, callback) {
            jQuery("div.ganttview-block", div).live("dblclick", function () {
                if (callback) { callback(jQuery(this).data("block-data") , jQuery(this), opts.cellWidth); }
            });
        }		
		
		
        function bindBlockResize(div, cellWidth, startDate, callback) {
        	jQuery("div.ganttview-block", div).resizable({
        		grid: cellWidth, 
        		handles: "e,w",
				opacity: 1 ,
        		stop:  function(event, ui) { //alert('ristretto');
        			var block = jQuery(this);
        			updateDataAndPosition(div, block, cellWidth, startDate);
        			if (callback) { callback(block.data("block-data")); }
        		}
        	});
        }
        
        function bindBlockDrag(div, cellWidth, startDate, callback) {
        	jQuery("div.ganttview-block", div).draggable({
/* 				revert: function()
				{
					var block = jQuery(this);

					var blockDataCopyed = {};
					jQuery.extend(true, blockDataCopyed, block.data("block-data"));

					updateDataAndPosition(div, block, cellWidth, startDate);

					var bloccato = true;

					if (callback) { bloccato = callback(block.data("block-data")  ); } 
							alert(bloccato);
					if(bloccato) //riportiamo l'oggetto alle condizione di default
						jQuery.extend(true, block.data("block-data"), blockDataCopyed);

					return true;
				}, */
        		axis: "x", 
        		grid: [cellWidth, cellWidth],
				opacity: 1 ,
        		stop: function (event, ui ) {
        			var block = jQuery(this);
        			updateDataAndPosition(div, block, cellWidth, startDate);
        			if (callback) { bloccato = callback(false, block.data("block-data") , block, ui.position, cellWidth  ); }
        		} 
        	});
        }
        
        function updateDataAndPosition(div, block, cellWidth, startDate) { 
        	var container = jQuery("div.ganttview-slide-container", div);
        	var scroll = container.scrollLeft();
			var offset = block.offset().left - container.offset().left - 1 + scroll; 
			
			// Set new start date
			var daysFromStart = Math.round(offset / cellWidth);
			var newStart = startDate.clone().addDays(daysFromStart);
			block.data("block-data").start = newStart;

			// Set new end date
        	var width = block.outerWidth();
			var numberOfDays = Math.round(width / cellWidth) - 1;
			block.data("block-data").end = newStart.clone().addDays(numberOfDays);
			jQuery("div.ganttview-block-text", block).text(numberOfDays + 1);
			
			// Remove top and left properties to avoid incorrect block positioning,
        	// set position to relative to keep blocks relative to scrollbar when scrolling
/* 			block.css("top", "").css("left", "")
				.css("position", "relative").css("margin-left", offset + "px"); */
        }
        
        return {
        	apply: apply	
        };
	}

    var ArrayUtils = 
	{
	
        contains: function (arr, obj) {
            var has = false;
            for (var i = 0; i < arr.length; i++) { if (arr[i] == obj) { has = true; } }
            return has;
        }
    };

	var DateUtils = 
	{
   
        isInInterval: function (date, startInterval, endInterval) 
		{
            if (!startInterval || !endInterval) { return false; }
			
            startInterval = Date.parse(startInterval); 
			endInterval = Date.parse(endInterval);
			
            if (startInterval.getYear() == 1901 || endInterval.getYear() == 8099) { return false; }
			
            if( date.compareTo(endInterval)<=0  &&   date.compareTo(startInterval)>=0) 
				return true;
			else
				return false;
        },   
		
		
        daysBetween: function (start, end) 
		{
            if (!start || !end) { return 0; }
            start = Date.parse(start); end = Date.parse(end);
            if (start.getYear() == 1901 || end.getYear() == 8099) { return 0; }
            var count = 0, date = start.clone();
            while (date.compareTo(end) == -1) { count = count + 1; date.addDays(1); }
            return count;
        },
        
        isWeekend: function (date) {
            return date.getDay() % 6 == 0;
        },

		getBoundaryDatesFromData: function (data, minDays) 
		{
/* 			var minStart = new Date();
			var maxEnd = new Date();	 */		
			
			var minStart = false;
			var maxEnd = false;
	
			for (var k = 0; k < data.length; k++) //add AZ
			{		
				for (var i = 0; i < data[k].series.length; i++) 
				{
					for (var j = 0; j < data[k].series[i].series.length; j++) 
					{
						for (var y = 0; y < data[k].series[i].series[j].series.length; y++) 
						{
							var start = Date.parse( data[k].series[i].series[j].series[y].start);
							var end = Date.parse(data[k].series[i].series[j].series[y].end)
							if (minStart==false)	{ minStart = start; }
							if (maxEnd==false)		{ maxEnd = end; }
							if (minStart.compareTo(start) == 1) { minStart = start; }
							if (maxEnd.compareTo(end) == -1) { maxEnd = end; }
						}
					}
					
				}
			}//end AZ		
			
			// Insure that the width of the chart is at least the slide width to avoid empty
			// whitespace to the right of the grid
			
 			if (DateUtils.daysBetween(minStart, maxEnd) < minDays) {
				maxEnd = minStart.clone().addDays(minDays);
			} 
			
			return [minStart, maxEnd];
		}
		
    }; 


})(jQuery);