/*
	object: 
		NtxGrid(idRootTable)
		
	Methods:
		appendRow()							aggiunge una nuova riga
		clear()								elimina tutte le righe
		deleteRow(index)					elimina la riga con indice index
		getCell(iRow, iCol)					recupera il contenuto di una cella distingue oggetti da testo
		getCellByName(iRow, colName)		recupera il contenuto di una cella distingue oggetti da testo
		getColCount()						recupera il numero di colonne inserite
		getColIndex(obj)					recupera l'indice di colonna di un oggetto in una cella
		getIndexColName(name)				recupera l'indice di colonna attraverso un nome
		getRowCount()						recupera il numero di righe inserite
		getRowIndex(obj)					recupera l'indice di righa di un oggetto in una cella
		setCell(iRow, iCol, value)			setta il contenuto di una cella distingue oggetti da testo
		setCellByName(iRow, colName, value)	setta il contenuto di una cella distingue oggetti da testo
		setCellClass(iRow, className)		setta la classe da accodare alla colonna di una cella
		setCells(iRow, record)				setta più celle tramite un record (oggetto le cui proprietà sono i campi)
		setColNames(names[])				setta i nomi delle colonne (serve con setCells() e con getIndexColName())
		setColSizes(sizes[])				setta le dimensioni di ogni colonna
		setTitles(titles[])					setta i titoli di ogni colonna
*/

function NtxGrid(idRootTable)
{
	this.rootTable = document.getElementById(idRootTable);
	
	this.rootTable.className = "NtxGridRootTable";
	
	var row = this.rootTable.insertRow(this.rootTable.rows.length);
	
	var cell = row.insertCell(0); 
	
	cell.className = "NtxGridTitleCell";
	
	this.title = document.createElement('table');
	
	this.title.className="NtxGridTitleTable";
	
	cell.appendChild(this.title);	
	
	row = this.rootTable.insertRow(this.rootTable.rows.length);
	
	cell = row.insertCell(0); 
	
	cell.className = "NtxGridBodyCell";

	var div = document.createElement('div');
	
	div.className = "NtxGridDivBodyCell";
	
	cell.appendChild(div);	
	
	this.body = document.createElement('table');
	
	this.body.className="NtxGridBodyTable";
	
	div.appendChild(this.body);	

	this.cellClasses = new Array();
	this.colNames = new Array();
	
	this.setColNames = function(names){
		this.colNames = names;
	}
	
	this.getIndexColName = function(name){
		var i;
		var len = this.colNames.length;

		for ( i=0; i<len; ++i )
			if (this.colNames[i] == name)
				return i;
		
		alert("colonna ["+name+"] non trovata in griglia!");

		return "";
	}
	
	this.setCells = function(iRow, record){
		var i;
		var len = this.colNames.length;

		for ( i=0; i<len; ++i )
		{
			var value = record[this.colNames[i]];
			
			if (value != undefined)
				this.setCell(iRow, i, value+"");
		}		
	}
	
	this.setColSizes = function(sizes){
		this.colSizes = sizes;
	}
	
	this.setCellClass = function(iRow, className){
		this.cellClasses[iRow] = className;
	}
	
	this.setTitles = function(titles){
		for(;this.title.rows.length>0;)
			this.title.deleteRow(0);
			
		var row = this.title.insertRow(this.title.rows.length);
			
		var i;
		var len = titles.length;
		var el;
		this.numColumns = len;
		
		for ( i=0; i<len; ++i )
		{
			el = document.createTextNode(titles[i]);

			var cell = row.insertCell(i); 
			
			cell.appendChild(el);	

			cell.className = "NtxGridTh";
			//cell.style.width = this.colSizes[i]+"px";
			
			
			//aggiunta zagatti
			var div = document.createElement("div");  
			div.style.width = this.colSizes[i]+"px";
			div.style.height = "1px"; 

			cell.appendChild(div); 
		}
	}
	
	this.getRowCount = function()
	{
		return this.body.rows.length;
	}
		
	this.getRowIndex = function(obj)
	{
		return obj.parentNode.parentNode.rowIndex;
	}
		
	this.getColIndex = function(obj)
	{
		return obj.parentNode.cellIndex;
	}

	this.deleteRow = function(index)
	{ 
		return this.body.deleteRow(index);
	}
	
	this.clear = function()
	{
		for(;this.body.rows.length>0;)
			this.body.deleteRow(0);
	}
	
	this.getCell = function(iRow, iCol)
	{
		var obj = this.body.rows[iRow].cells[iCol].childNodes[0];
		
		if ( obj.nodeValue == null )
			return obj;
		
		return obj.nodeValue;
	}

	this.getCellByName = function(iRow, colName)
	{
		return this.getCell(iRow, this.getIndexColName(colName));
	}

	
	this.setCell = function(iRow, iCol, value)
	{
		var cell = this.body.rows[iRow].cells[iCol]; 

		if( typeof value == "string" )
			value = document.createTextNode(value);
		else if ( value.type=='textarea' || value.type=='text' || value.type=='select-one' )
			value.style.width = (this.colSizes[iCol])+"px";  
		
		cell.removeChild(cell.lastChild);
		cell.appendChild(value);
	}	 

	this.setCellByName = function(iRow, colName, value)
	{
		this.setCell(iRow, this.getIndexColName(colName), value);
	}
	
	this.getColCount = function()
	{
		return this.numColumns;
	}
	
	this.appendRow = function()
	{
		var row = this.body.insertRow(this.body.rows.length);
		var i;
		var el;

		for ( i=0; i<this.numColumns; ++i )
		{
			el = document.createTextNode("");
			
			var cell = row.insertCell(i); 
	
			cell.appendChild(el);

			if ( typeof this.cellClasses[i] == "string" )
				cell.className = "NtxGridTd "+this.cellClasses[i];
			else
				cell.className = "NtxGridTd"; 		 
		}
	}
}