/*
 * 
 * Copyright (c) 2015 Natisoft S.r.l.
 * Licensed under the MIT license.
 * 
 * @Author SZ <zingaro.natisoft@gmail.com>
 * @CreationDate 18/11/2015
 */

/*
 * Module for report management ManProNet
 * @PlotData - the matrix of data to be plotted
 * @PlotDiv - the div container of the plot
 * @Stack - true if you want variables to be stacked on each other
 * @Width - value of the width of the tick
 * @Labeler - container for legend
 * @Chart - support only pie chart or bars chart
 * @Mode - null or time mode 
 * @Labels - array of labels in the form [["category1","label1"],["category2","label2"],....]
 */

function ReportManProNet(PlotData, PlotDiv, Stack, Width, Labeler, Chart, Mode, Labels) {
    this.PlotData = PlotData;
    this.PlotDiv = PlotDiv;
    this.Stack = Stack;
    this.Width = Width;
    this.Labeler = Labeler;
    this.Chart = Chart;
    this.Mode = Mode;
    this.Labels = Labels;
}

/*
 * SendPlot function actually call plot from flot jquery with the parameter 
 * passed from the html/php or whatever
 */
ReportManProNet.prototype.SendPlot = function () {
    var Columns = this.PlotData.length;
    if (this.PlotData.length > 6) {
        Columns = Math.round((Columns / 2) - 1);
    }
    var bars = false;
    var pie = false;
    if (this.Chart === 'bars') {
        bars = true;
    }
    if (this.Chart === 'pie') {
        pie = true;
    }
    var plot = $.plot(this.PlotDiv, this.PlotData, GenerateOptions(Math.round(Columns), this.Stack, this.Width, this.Labeler, bars, pie, this.Mode, this.Labels));
    
    return plot;
}
;
/*
 * GenerateOptions function support SendPlot building the options variable for 
 * plot to display
 * @columns : is automatically generated taking in account the number of 
 * column of the @PlotData matrix
 * @mode : [time, ...]
 * the idea is to take a general options generator that work with different
 * plots, as parameters change
 */
function GenerateOptions(columns, stack, barWidth, labeler, bars, pie, mode, labels) {
    var options = {
        xaxis: {
            mode: mode,
            ticks: labels,
            tickLength: 0,
            minTickSize: [1, "month"],
            timeformat: "%b'%y",
            monthNames: ["Gen", "Feb", "Mar", "Apr", "Mag", "Giu", "Lug", "Ago", "Set", "Ott", "Nov", "Dic"],
            autoscaleMargin: 0.01
        },
        yaxis: {
            autoscaleMargin: 0.1
        },
        series: {
            stack: stack,
            bars: {
                show: bars,
                barWidth: barWidth
            },
            pie: {
                show: pie,
                radius: 1,
                label: {
                    show: true,
                    radius: 2 / 3,
                    threshold: 0.05,
                    formatter: labelFormatter,
                    background: {
                        opacity: 0.5
                    }
                }
            }
        },
        grid: {
            hoverable: true,
            clickable: true,
            borderWidth: 0
        },
        legend: {
            show: true,
            noColumns: columns,
            backgroundColor: "#EEE",
            labelBoxBorderColor: "none",
            container: labeler
        },
        tooltip: {
            show: true,
            content: "N&deg; Ticket %s: %y"
        },
        colors: ['#00C90D', '#FF4040', '#4188D2', '#AD66D5', '#000000', '#FFB840', '#620012', '#00CCFF']
    };
    return options;
}

function labelFormatter(label, series) {
    return "<div style='font-size:10pt; font-weight: 500; text-align:center; padding:2px; color:white;'>" + label + "<br/>" + Math.round(series.percent) + "%</div>";
}
